# frozen_string_literal: true

require "digest/sha2"
require "feen/dumper"
require "qi"
require "ugoki/position"

module Romance
  class Position < ::Ugoki::Position
    attr_reader :positions

    def after(move)
      hash = ::Qi.call(move, square: square, in_hand: in_hand)
      self.class.new(shape: shape, side_id: next_side_id, **hash)
    end

    def ally_king_on_the_board?
      ally_pieces.values.any? { |piece_name| piece_name.include?("-") }
    end

    # The list of pieces on the board owned by the current player, with squares.
    #
    # @return [Hash] Top-side's pieces if turn to topside, bottom-side's ones
    #   otherwise.
    def ally_pieces
      turn_to_topside? ? square_topside_pieces : square_bottomside_pieces
    end

    def next_side_id
      side_id.next % players_count
    end

    def players_count
      2
    end

    def sha256
      ::Digest::SHA256.hexdigest(to_feen)
    end

    def to_feen
      ::FEEN::Dumper.call(**to_h)
    end

    def to_h
      {
        side_id: side_id,
        square:  square,
        shape:   shape,
        in_hand: in_hand
      }
    end

    # @note Faster than FEEN to calculate.
    def to_s
      [
        side_id,
        square.keys.sort.inject("") { |s, key| s + "#{key}:#{square.fetch(key)}" },
        shape.join("x"),
        in_hand.sort.join(";")
      ].join("--")
    end
  end
end
